import uuid

from datetime import date, datetime

from barker.models.guest_book_status import GuestBookStatus
from pydantic import (
    BaseModel,
    ConfigDict,
    Field,
    field_serializer,
    field_validator,
    model_validator,
)

from . import to_camel


class GuestBookIn(BaseModel):
    name: str
    phone: str
    address: str | None = None
    pax: int = Field(ge=0)
    booking_date: datetime | None = None
    arrival_date: datetime | None = None
    notes: str | None = None
    type_: GuestBookStatus

    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)

    @field_validator("booking_date", mode="before")
    @classmethod
    def parse_booking_date(cls, value):
        if value is None:
            return None
        if isinstance(value, datetime):
            return value
        return datetime.strptime(value, "%d-%b-%Y %H:%M")

    @field_serializer("booking_date")
    def serialize_booking_date(self, value: datetime | None, _info):
        return None if value is None else value.strftime("%d-%b-%Y %H:%M")

    @field_validator("arrival_date", mode="before")
    @classmethod
    def parse_arrival_date(cls, value):
        if value is None:
            return None
        if isinstance(value, datetime):
            return value
        return datetime.strptime(value, "%d-%b-%Y %H:%M")

    @field_serializer("arrival_date")
    def serialize_arrival_date(self, value: datetime | None, _info):
        return None if value is None else value.strftime("%d-%b-%Y %H:%M")

    @model_validator(mode="after")
    def nulls(self) -> "GuestBookIn":
        if self.arrival_date is None and self.booking_date is None:
            raise ValueError("Both arrival and booking date cannot be null")
        return self


class GuestBook(GuestBookIn):
    id_: uuid.UUID
    last_edit_date: datetime

    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)

    @field_validator("last_edit_date", mode="before")
    @classmethod
    def parse_last_edit_date(cls, value):
        if isinstance(value, datetime):
            return value
        return datetime.strptime(value, "%d-%b-%Y %H:%M")

    @field_serializer("last_edit_date")
    def serialize_last_edit_date(self, value: datetime, _info):
        return value.strftime("%d-%b-%Y %H:%M")


class GuestBookListItem(BaseModel):
    id_: uuid.UUID
    serial: int
    count: int
    name: str
    phone: str
    pax: int
    booking_date: datetime | None = None
    arrival_date: datetime | None = None
    last_edit_date: datetime

    status: GuestBookStatus
    table_id: uuid.UUID | None = None
    voucher_id: uuid.UUID | None = None
    table_name: str | None = None

    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)

    @field_validator("booking_date", mode="before")
    @classmethod
    def parse_booking_date(cls, value):
        if value is None:
            return None
        if isinstance(value, datetime):
            return value
        return datetime.strptime(value, "%d-%b-%Y %H:%M")

    @field_serializer("booking_date")
    def serialize_booking_date(self, value: datetime | None, _info):
        return None if value is None else value.strftime("%d-%b-%Y %H:%M")

    @field_validator("arrival_date", mode="before")
    @classmethod
    def parse_arrival_date(cls, value):
        if value is None:
            return None
        if isinstance(value, datetime):
            return value
        return datetime.strptime(value, "%d-%b-%Y %H:%M")

    @field_serializer("arrival_date")
    def serialize_arrival_date(self, value: datetime | None, _info):
        return None if value is None else value.strftime("%d-%b-%Y %H:%M")

    @field_validator("last_edit_date", mode="before")
    @classmethod
    def parse_last_edit_date(cls, value):
        if isinstance(value, datetime):
            return value
        return datetime.strptime(value, "%d-%b-%Y %H:%M")

    @field_serializer("last_edit_date")
    def serialize_last_edit_date(self, value: datetime, _info):
        return value.strftime("%d-%b-%Y %H:%M")


class GuestBookList(BaseModel):
    date_: date
    list: list[GuestBookListItem]
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)

    @field_validator("date_", mode="before")
    @classmethod
    def parse_date(cls, value):
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("date_")
    def serialize_date(self, value: date, _info):
        return value.strftime("%d-%b-%Y")
